#!/bin/bash

# spdx-license-identifier: gpl-3.0-only
# Copyright (C) 2022 Michał Góral

[[ -n "$LIBGPAK_BUILD_PACKAGES_SH" ]] && return
LIBGPAK_BUILD_PACKAGES_SH=1

source "${LIBRARY}/message.sh"
source "${LIBRARY}/path.sh"
source "${LIBRARY}/exit_codes.sh"

makedeb_local() {
    local distro="$1"
    local pkgdir="$2"
    local pkgname=$(basename -- "$pkgdir")

    (
        export BUILDDIR="${BUILD_DIR}/${distro}/.build/${pkgname}/build"
        export SRCDEST="${BUILD_DIR}/${distro}/.build/${pkgname}/src"
        export SRCPKGDEST="${BUILD_DIR}/${distro}/.build/${pkgname}/srcpkg"
        export PKGDEST="${BUILD_DIR}/${distro}/${pkgname}"
        export GPAK_DISTRO="${distro}"

        cd "${pkgdir}"
        makedeb --no-confirm --pass-env
    )
}

makedeb_docker() {
    local distro="$1"
    local pkgdir="$2"
    local pkgname=$(basename -- "$pkgdir")
    local container="gpak:${distro}"
    local outdir="$(make_absolute_path "${BUILD_DIR}/${distro}/${pkgname}")"

    local contname="gpak_${distro}_${pkgname}"
    local contid=$(docker container ls --all --quiet --filter="name=${contname}")

    if [[ -n "${contid}" ]] && (( ! CACHE )); then
        docker  container rm "${contid}"
        unset contid
    fi

    if [[ -z "${contid}" ]]; then
        contargs=(
            --name "${contname}"
            -v "${pkgdir}:/script"
            -v "${outdir}:/out"
            -e "BUILDDIR=/build/${pkgname}/build"
            -e "SRCDEST=/build/${pkgname}/sources"
            -e "SRCPKGDEST=/build/${pkgname}/srcpkg"
            -e "PKGDEST=/out"
            -e "GPAK_DISTRO=${distro}"
            --workdir="/script"
        )
        (( ! CACHE )) && contargs+=("--rm")

        makedebargs=( makedeb --sync-deps --no-confirm --pass-env )
        if [[ -e "${pkgdir}/PKGBUILD.${distro}" ]]; then
            msg2 "Using distro-specific PKGBUILD.%s" "${distro}"
            makedebargs+=( "--file" "PKGBUILD.${distro}" )
        fi

        contargs+=(
            "${container}"
            "${makedebargs[@]}"
        )

        contid=$(docker container create "${contargs[@]}")
    fi

    mkdir -p "${outdir}"
    docker start -a "${contid}"
}

build_distro_packages() {
    local distro="$1" ; shift
    local packages=("$@")
    local ret=0

    msg "Building packages for %s" "$distro"
    for pkg in "${packages[@]}"; do
        if [[ -e "${pkg}/PKGBUILD" ]] || [[ -e "${pkg}/PKGBUILD.${distro}" ]]; then
            msg2 "Building %s..." "$pkg"
            if (( BUILD_LOCAL )); then
                makedeb_local "$distro" "$pkg"
                [[ $? -ne 0 ]] && ret=$E_BUILD_ERROR
            else
                makedeb_docker "$distro" "$pkg"
                [[ $? -ne 0 ]] && ret=$E_BUILD_ERROR
            fi
        else
            error "Package doesn't have PKGBUILD file: %s" "$pkg"
            ret=$E_BUILD_ERROR
        fi
    done

    return $ret
}

build_packages() {
    local req_packages=(${@/#/"${PACKAGES_DIR}/"})

    for distro in "${DISTRIBUTIONS[@]}"; do
        build_distro_packages "$distro" "${req_packages[@]}"
        (( $? )) && exit $E_ERROR
    done
}
